### **研究整个矩阵的误差（全局误差分析）**

为了分析整个矩阵的重构误差 $A - A_\kappa$，我们可以采用不同的矩阵范数来衡量误差的大小。常见的选择包括：

1. **Frobenius 范数（F-范数）**：衡量矩阵所有元素的平方和，适用于整体误差分析。
2. **谱范数（2-范数）**：衡量矩阵的最大奇异值，适用于最坏情况下的误差分析。
3. **核范数（Trace 范数）**：衡量矩阵的奇异值之和，适用于低秩矩阵分析。

---

### **1. Frobenius 范数（F-范数）分析**
F-范数定义为：
$$
\| A - A_\kappa \|_F = \sqrt{\sum_{i,j} |(A - A_\kappa)_{ij}|^2}
$$
由于我们已经知道：
$$
A - A_\kappa = \sum_{m=1}^r \Delta \lambda_m x_m x_m^T + \sum_{m=\kappa+1}^r \widetilde{\lambda}_m x_m x_m^T
$$
我们可以计算其 F-范数：
$$
\| A - A_\kappa \|_F^2 = \left\| \sum_{m=1}^r \Delta \lambda_m x_m x_m^T + \sum_{m=\kappa+1}^r \widetilde{\lambda}_m x_m x_m^T \right\|_F^2
$$
由于 $x_m$ 是正交归一化的（假设 $x_i^T x_j = \delta_{ij}$），交叉项消失：
$$
\| A - A_\kappa \|_F^2 = \sum_{m=1}^r \Delta \lambda_m^2 \| x_m x_m^T \|_F^2 + \sum_{m=\kappa+1}^r \widetilde{\lambda}_m^2 \| x_m x_m^T \|_F^2
$$
而 $\| x_m x_m^T \|_F = \sqrt{\text{tr}(x_m x_m^T x_m x_m^T)} = \sqrt{\text{tr}(x_m x_m^T)} = \| x_m \|_2 = 1$，因此：
$$
\| A - A_\kappa \|_F^2 = \sum_{m=1}^r \Delta \lambda_m^2 + \sum_{m=\kappa+1}^r \widetilde{\lambda}_m^2
$$
最终：
$$
\| A - A_\kappa \|_F \leq \sqrt{ \sum_{m=1}^r |\Delta \lambda_m|^2 + \sum_{m=\kappa+1}^r |\widetilde{\lambda}_m|^2 }
$$
**结论**：
- F-范数误差由特征值偏差和截断特征值的平方和决定。
- 如果 $\Delta \lambda_m$ 和 $\widetilde{\lambda}_m$ 较小，F-范数误差也会较小。

---

### **2. 谱范数（2-范数）分析**
谱范数定义为：
$$
\| A - A_\kappa \|_2 = \sigma_{\max}(A - A_\kappa)
$$
其中 $\sigma_{\max}$ 是最大奇异值。由于 $A - A_\kappa$ 是对称矩阵，其谱范数等于最大绝对特征值：
$$
\| A - A_\kappa \|_2 = \max \left\{ |\Delta \lambda_1|, \dots, |\Delta \lambda_r|, |\widetilde{\lambda}_{\kappa+1}|, \dots, |\widetilde{\lambda}_r| \right\}
$$
**结论**：
- 谱范数误差由最大的特征值偏差或截断特征值决定。
- 如果所有 $\Delta \lambda_m$ 和 $\widetilde{\lambda}_m$ 都较小，谱范数误差也会较小。

---

### **3. 核范数（Trace 范数）分析**
核范数定义为：
$$
\| A - A_\kappa \|_* = \sum_{i=1}^r \sigma_i (A - A_\kappa)
$$
由于 $A - A_\kappa$ 的奇异值就是 $|\Delta \lambda_1|, \dots, |\Delta \lambda_r|, |\widetilde{\lambda}_{\kappa+1}|, \dots, |\widetilde{\lambda}_r|$，因此：
$$
\| A - A_\kappa \|_* = \sum_{m=1}^r |\Delta \lambda_m| + \sum_{m=\kappa+1}^r |\widetilde{\lambda}_m|
$$
**结论**：
- 核范数误差等于特征值偏差和截断特征值的绝对值和。
- 这与逐元素误差的界一致，但核范数衡量的是全局误差。

---